// $Id: ConnectionDialog.cpp,v 1.21 2014/10/02 15:19:45 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ConnectionDialog.h"
#include "ConnectionDialog.moc"
#include "AnimatedLineEditor.h"
#include "AnimatedTreeView.h"
#include "GridLayout.h"
#include "IconEngine.h"
#include "IconNames.h"
#include "InformationDialog.h"
#include "QuestionDialog.h"
#include "TunnelDialog.h"

#include <QLabel>
#include <QLayout>

//___________________________________________________________
ConnectionDialog::ConnectionDialog( QWidget* parent ):
CustomDialog(parent, OkButton|CancelButton )
{

    Debug::Throw( "ConnectionDialog::ConnectionDialog.\n" );

    setOptionName( "CONNECTION_DIALOG" );

    // grid layout
    GridLayout* gridLayout( new GridLayout() );
    gridLayout->setMargin(0);
    gridLayout->setSpacing(5);
    gridLayout->setMaxCount(2);
    gridLayout->setColumnAlignment(0, Qt::AlignRight|Qt::AlignVCenter );
    mainLayout().addLayout( gridLayout );

    // name
    QLabel* label;
    gridLayout->addWidget( label = new QLabel( tr( "Name:" ), this ) );
    gridLayout->addWidget( nameEditor_ = new AnimatedLineEditor( this ) );
    label->setBuddy( nameEditor_ );

    gridLayout->addWidget( label = new QLabel( tr( "Host:" ), this ) );
    gridLayout->addWidget( hostEditor_ = new AnimatedLineEditor( this ) );
    label->setBuddy( hostEditor_ );

    gridLayout->addWidget( label = new QLabel( tr( "User:" ), this ) );
    gridLayout->addWidget( userEditor_ = new AnimatedLineEditor( this ) );
    label->setBuddy( userEditor_ );

    gridLayout->addWidget( label = new QLabel( tr( "Options:" ), this ) );
    gridLayout->addWidget( optionsEditor_ = new AnimatedLineEditor( this ) );
    label->setBuddy( optionsEditor_ );

    mainLayout().addWidget( autoConnect_ = new QCheckBox( tr( "Connect at startup" ), this ) );
    autoConnect_->setToolTip( tr( "Check this option to start this connection automatically at startup" ) );

    // separator
    QFrame* frame = new QFrame( this );
    frame->setFrameStyle( QFrame::HLine | QFrame::Sunken );
    mainLayout().addWidget( frame );

    mainLayout().addWidget( new QLabel( tr( "Tunnels:" ) ) );

    // horizontal layout
    QHBoxLayout* hLayout = new QHBoxLayout();
    hLayout->setMargin(0);
    hLayout->setSpacing(5);
    mainLayout().addLayout( hLayout );

    hLayout->addWidget( list_ = new AnimatedTreeView( this ) );
    _list().setModel( &model_ );
    _list().setSortingEnabled( false );

    QVBoxLayout* vLayout = new QVBoxLayout();
    vLayout->setMargin(0);
    vLayout->setSpacing(5);
    hLayout->addLayout( vLayout );

    // buttons
    vLayout->addWidget( addButton_ = new QPushButton( IconEngine::get( IconNames::Add ), tr( "Add" ), this ) );
    vLayout->addWidget( removeButton_ = new QPushButton( IconEngine::get( IconNames::Remove ), tr( "Remove" ), this ) );
    vLayout->addWidget( editButton_ = new QPushButton( IconEngine::get( IconNames::Edit ), tr( "Edit" ), this ) );
    vLayout->addStretch( 1 );

    connect( addButton_, SIGNAL(pressed()), SLOT(_add()) );
    connect( removeButton_, SIGNAL(pressed()), SLOT(_remove()) );
    connect( editButton_, SIGNAL(pressed()), SLOT(_edit()) );
    connect( &_list(), SIGNAL(activated(QModelIndex)), SLOT(_edit()) );

    connect( _list().selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), SLOT(_updateButtons()) );
    connect( _list().selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)), SLOT(_updateButtons()) );

    okButton().setText( tr( "Save Connection" ) );
    okButton().setIcon( IconEngine::get( IconNames::Save ) );

    _updateButtons();

}


//________________________________________
void ConnectionDialog::setConnectionAttributes( const ConnectionAttributes& attributes )
{
    Debug::Throw( "ConnectionDialog::setConnectionAttributes.\n" );

    // update editors
    nameEditor_->setText( attributes.name() );
    hostEditor_->setText( attributes.host() );
    userEditor_->setText( attributes.user() );
    optionsEditor_->setText( attributes.options() );
    autoConnect_->setChecked( attributes.autoConnect() );

    // update model
    _model().set( attributes.tunnelAttributes() );
    _list().resizeColumns();


}

//________________________________________
ConnectionAttributes ConnectionDialog::connectionAttributes( void ) const
{
    Debug::Throw( "ConnectionDialog::connectionAttributes.\n" );
    ConnectionAttributes out;
    out.setName( nameEditor_->text() );
    out.setHost( hostEditor_->text() );
    out.setUser( userEditor_->text() );
    out.setOptions( optionsEditor_->text() );
    out.setAutoConnect( autoConnect_->isChecked() );
    out.setTunnelAttributes( _model().get() );

    return out;
}

//________________________________________
void ConnectionDialog::_updateButtons( void )
{
    Debug::Throw( "ConnectDialog::_updateButtons.\n" );

    bool has_selection( !_list().selectionModel()->selectedRows().empty() );
    editButton_->setEnabled( has_selection );
    removeButton_->setEnabled( has_selection );

}

//________________________________________
void ConnectionDialog::_add( void )
{

    Debug::Throw( "ConnectionDialog::_add.\n" );
    TunnelAttributes attributes;
    while( !attributes.isValid() )
    {

        // create dialog
        TunnelDialog dialog( this );
        dialog.setTunnelAttributes( attributes );
        if( !dialog.centerOnParent().exec() ) return;

        // get attributes and check validity
        attributes = dialog.tunnelAttributes();
        if( !attributes.isValid() )
        {
            const QString buffer = QString( tr( "Tunnel settings are invalid: \n%1\nModify ?" ) ).arg( attributes.errors().join( "\n" ) );
            if( !QuestionDialog( this, buffer ).centerOnWidget( window() ).exec() )
            { break; }
        }
    }

    // add to model
    _model().add( attributes );

}

//________________________________________
void ConnectionDialog::_remove( void )
{

    Debug::Throw( "ConnectionDialog::_remove.\n" );

    // retrieve selected items; retrieve only recordable options
    TunnelAttributesModel::List selection( _model().get( _list().selectionModel()->selectedRows() ) );
    TunnelAttributesModel::List removed;
    for( TunnelAttributesModel::List::const_iterator iter = selection.begin(); iter != selection.end(); ++iter )
    { removed << *iter; }

    // remove
    _model().remove( removed );
    _list().resizeColumns();

}

//________________________________________
void ConnectionDialog::_edit( void )
{

    Debug::Throw( "ConnectionDialog::_edit.\n" );
    QModelIndex current( _list().selectionModel()->currentIndex() );
    if( !current.isValid() )
    {
        InformationDialog( this, tr( "No item selected. <Edit> canceled." ) ).exec();
        return;
    }

    // retrieve DocumentClass matching name
    TunnelAttributes attributes( _model().get( current ) );
    TunnelDialog dialog( this );
    dialog.setTunnelAttributes( attributes );
    if( !dialog.centerOnParent().exec() ) return;

    // remove old attributes
    _model().remove( attributes );

    // add new attributes
    attributes = dialog.tunnelAttributes();

    // check validity
    if( !attributes.isValid() )
    {
        const QString buffer = QString( tr( "Tunnel settings are invalid:\n%1" ) ).arg( attributes.errors().join( "\n" ) );
        InformationDialog( this, buffer ).centerOnWidget( window() ).exec();
    }

    // add to model_
    _model().add( attributes );

    // make sure item is selected
    QModelIndex index( _model().index( attributes ) );
    if( index != _list().selectionModel()->currentIndex() )
    {
        _list().selectionModel()->select( index,  QItemSelectionModel::Clear|QItemSelectionModel::Select|QItemSelectionModel::Rows );
        _list().selectionModel()->setCurrentIndex( index,  QItemSelectionModel::Current|QItemSelectionModel::Rows );
    }

    _list().resizeColumns();

}
